#include <sync_adapter.h>
#include "sync-adapter-service-app.h"


int account_id = 0;


/*
 * Callback for receiving the result of sync start operation
 * It is invoked in the following cases
 * - sync_manager_on_demand_sync_job() which was called in Sync Adapter is complete
 * - In the case of using sync_manager_add_periodic_sync_job() in Sync Adapter,
 *   it is called after some times which is set as period interval later
 * - In the case of using sync_manager_add_data_change_sync_job() in Sync Adapter,
 *   it is called when status change is detected in DB for specific capability
 */
static bool
handle_start_sync(account_h account, const char *sync_job_name, const char *sync_capability, bundle *sync_job_user_data)
{
	dlog_print(DLOG_INFO, LOG_TAG, "handle_start_sync() is invoked in service application");

	/* Account handle which is transfered from the Sync Adapter */
	if (account) {
		account_get_account_id(account, &account_id);
		dlog_print(DLOG_INFO, LOG_TAG, "Account ID [%d]", account_id);
	} else {
		dlog_print(DLOG_INFO, LOG_TAG, "Accountless case");
	}

	if (sync_job_name)
		dlog_print(DLOG_INFO, LOG_TAG, "Sync Job Name [%s]", sync_job_name);
	else
		dlog_print(DLOG_INFO, LOG_TAG, "Data Change Sync Job Capability [%s]", sync_capability);

	app_control_h app_control;
	int ret = app_control_create(&app_control);
	ret = app_control_set_app_id(app_control, SYNC_ADAPTER_APP_ID);
	if (ret == APP_CONTROL_ERROR_NONE)
		dlog_print(DLOG_INFO, LOG_TAG, "Sync Adapter application ID is set as [%s]", SYNC_ADAPTER_APP_ID);
	else
		dlog_print(DLOG_INFO, LOG_TAG, "app_control_set_app_id() is failed [%d : %s]", ret, get_error_message(ret));

	/*
	 * If there is sync_capability instead of sync_job_name,
	 * this calling is for notifying data change sync operation is over to Sync Adapter Service
	 */
	bool is_data_change = sync_capability ? true : false;

	if (is_data_change) {
		ret = app_control_set_operation(app_control, "http://tizen.org/appcontrol/operation/data_change_sync_complete");
	} else {
		/*
		 * If there is sync_job_name instead of sync_capability,
		 * this calling is for notifying on demand or periodic sync operation is over to Sync Adapter Service
		 */
		bool is_periodic = !strcmp(sync_job_name, "Periodic") ? true : false;
		if (is_periodic)
			ret = app_control_set_operation(app_control, "http://tizen.org/appcontrol/operation/periodic_sync_complete");
		else
			ret = app_control_set_operation(app_control, "http://tizen.org/appcontrol/operation/on_demand_sync_complete");
	}

	/*
	 * Insert your code for data synchronization between this application with server
	 * For this application, Deliver the result of sync operation to Sync Adapter
	 * for verifying proper operation of sync-manager APIs by using App Control handle
	 */
	app_control_send_launch_request(app_control, NULL, NULL);

	dlog_print(DLOG_INFO, LOG_TAG, "Sync start operation is completed");

	return true;
}


/*
 * Callback for receiving the result of sync stop operation
 */
static void
handle_stop_sync(account_h account, const char *sync_job_name, const char *sync_capability, bundle *sync_job_user_data)
{
	dlog_print(DLOG_INFO, LOG_TAG, "handle_stop_sync() is invoked in service application");

	if (account) {
		account_get_account_id(account, &account_id);
		dlog_print(DLOG_INFO, LOG_TAG, "Account ID [%d]", account_id);
	} else {
		dlog_print(DLOG_INFO, LOG_TAG, "Accountless case");
	}

	if (sync_job_name)
		dlog_print(DLOG_INFO, LOG_TAG, "Sync Job Name [%s]", sync_job_name);
	else
		dlog_print(DLOG_INFO, LOG_TAG, "Data Change Sync Job Capability [%s]", sync_capability);

	/*
	 * Insert your code for the case of canceling sync operation
	 * This callback can receive the result of what sync jobs is canceled
	 */

	dlog_print(DLOG_INFO, LOG_TAG, "Sync stop operation is complete");
}


/*
 * App create callback
 */
bool
service_app_create(void *data)
{
	dlog_print(DLOG_INFO, LOG_TAG, "Sync Adapter Service : service_app_create is invoked");
	sync_adapter_set_callbacks(handle_start_sync, handle_stop_sync);
	return true;
}


/*
 * App terminate callback
 */
void
service_app_terminate(void *data)
{
	dlog_print(DLOG_INFO, LOG_TAG, "Sync Adapter Service : service_app_terminate is invoked");
	return;
}


/*
 * App control callback
 */
void
service_app_control(app_control_h app_control, void *data)
{
	dlog_print(DLOG_INFO, LOG_TAG, "Sync Adapter Service : service_app_control is invoked");

	char *app_id = NULL;
	int ret = app_control_get_app_id(app_control, &app_id);
	if (ret != APP_CONTROL_ERROR_NONE) {
		dlog_print(DLOG_INFO, LOG_TAG, "app_control_get_app_id() is failed [%d : %s]", ret, get_error_message(ret));
		return;
	}

	if (app_id != NULL) {
		dlog_print(DLOG_INFO, LOG_TAG, "Caller application is [%s]", app_id);
		free(app_id);
	}

	return;
}


/*
 * App low battery callback
 */
static void
service_app_low_battery(app_event_info_h event_info, void *user_data)
{
	/* APP_EVENT_LOW_BATTERY */
}


/*
 * App low memory callback
 */
static void
service_app_low_memory(app_event_info_h event_info, void *user_data)
{
	/* APP_EVENT_LOW_MEMORY */
}


/*
 * App orientation changed callback
 */
static void
service_app_orient_changed(app_event_info_h event_info, void *user_data)
{
	/* APP_EVENT_DEVICE_ORIENTATION_CHANGED */
	return;
}


/*
 * App language changed callback
 */
static void
service_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/* APP_EVENT_LANGUAGE_CHANGED */
	return;
}


/*
 * App region format changed callback
 */
static void
service_app_region_changed(app_event_info_h event_info, void *user_data)
{
	/* APP_EVENT_REGION_FORMAT_CHANGED */
}


/*
 * Sync Adapter Service main()
 */
int main(int argc, char *argv[])
{
	char ad[50] = {0,};
	service_app_lifecycle_callback_s event_callback;
	app_event_handler_h handlers[5] = {NULL, };

	/* Set event callback functions */
	event_callback.create = service_app_create;
	event_callback.terminate = service_app_terminate;
	event_callback.app_control = service_app_control;

	/* Set low battery event */
	service_app_add_event_handler(&handlers[APP_EVENT_LOW_BATTERY], APP_EVENT_LOW_BATTERY, service_app_low_battery, &ad);
	/* Set low memory event */
	service_app_add_event_handler(&handlers[APP_EVENT_LOW_MEMORY], APP_EVENT_LOW_MEMORY, service_app_low_memory, &ad);
	/* Set orientation changed event */
	service_app_add_event_handler(&handlers[APP_EVENT_DEVICE_ORIENTATION_CHANGED], APP_EVENT_DEVICE_ORIENTATION_CHANGED, service_app_orient_changed, &ad);
	/* Set language changed event */
	service_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED], APP_EVENT_LANGUAGE_CHANGED, service_app_lang_changed, &ad);
	/* Set region format changed event */
	service_app_add_event_handler(&handlers[APP_EVENT_REGION_FORMAT_CHANGED], APP_EVENT_REGION_FORMAT_CHANGED, service_app_region_changed, &ad);

	return service_app_main(argc, argv, &event_callback, ad);
}
